package com.textura.cpms.service.impl

import javax.ejb.Local
import javax.ejb.Stateless
import javax.ejb.TransactionAttribute
import javax.persistence.EntityManager
import javax.persistence.PersistenceContext
import javax.persistence.Query

import org.slf4j.Logger
import org.slf4j.LoggerFactory

import com.textura.cpms.model.Person
import com.textura.cpms.model.PersonReference
import com.textura.cpms.service.PersonManager
import com.textura.cpms.service.QueryMetaData

@Local
@TransactionAttribute
@Stateless
class PersonManagerEJB implements PersonManager {

	static final Logger logger = LoggerFactory.getLogger(PersonManagerEJB)
	
	@PersistenceContext
	protected EntityManager entityManager
	
	@Override
//	@TransactionAttribute(javax.ejb.TransactionAttributeType.SUPPORTS)
	public List<PersonReference> findPersonReferences(QueryMetaData queryMetaData, String... propertyNames) {
		
		def pql = new PersonQueryBuilder(queryMetaData, propertyNames).build()
		
		logger.debug('  Using PQL: {}', pql)
		
		Query query = entityManager.createQuery(pql)
		if (queryMetaData.propertyName && queryMetaData.searchTerm) {
			query.setParameter("searchTerm", queryMetaData.searchTerm)
		}
		
		List<Object[]> results = query.getResultList();
		logger.debug('  Received {} records', results.size())
		
		def references = results.inject([]) { list, Object[] row ->
			// ID is always included
			assert row.size() == propertyNames.size() + 1, 'invalid number of returned values'
			
			logger.debug('  Adding: {}', row as List)
			
			def valueMap = [:]
			row[1..-1].eachWithIndex { property, i ->	
				valueMap[propertyNames[i]] = property
			}
			
			list << new PersonReference(row[0], valueMap)
		}
		
		return references
	}
	
//	@TransactionAttribute(javax.ejb.TransactionAttributeType.SUPPORTS)
	@Override
	Person loadPerson(Long id) {
        return entityManager.find(Person.class, id);
	}

	@Override
	Person savePerson(Person person) {
		
		logger.info('Saving {}', person)
		
		if (person.getId() == null) {
			entityManager.persist(person)
		}
		else {
			entityManager.merge(person)
		}

		return person
	}
}
