package com.textura.cpms.web.addressbook

import javax.ejb.EJB
import javax.servlet.ServletException
import javax.servlet.annotation.WebServlet
import javax.servlet.http.HttpServletRequest

import org.slf4j.Logger
import org.slf4j.LoggerFactory

import com.textura.cpms.model.Person
import com.textura.cpms.service.PersonManager
import com.textura.cpms.service.QueryMetaData
import com.textura.cpms.service.SearchFilter
import com.textura.cpms.web.addressbook.data.PersonReferenceContainer
import com.textura.cpms.web.addressbook.ui.HelpWindow
import com.textura.cpms.web.addressbook.ui.ListView
import com.textura.cpms.web.addressbook.ui.NavigationTree
import com.textura.cpms.web.addressbook.ui.PersonForm
import com.textura.cpms.web.addressbook.ui.PersonList
import com.textura.cpms.web.addressbook.ui.SearchView
import com.textura.cpms.web.addressbook.ui.SharingOptions
import com.vaadin.Application
import com.vaadin.data.Property
import com.vaadin.data.Property.ValueChangeEvent
import com.vaadin.data.Property.ValueChangeListener
import com.vaadin.event.ItemClickEvent
import com.vaadin.event.ItemClickEvent.ItemClickListener
import com.vaadin.terminal.Sizeable
import com.vaadin.terminal.ThemeResource
import com.vaadin.terminal.gwt.server.AbstractApplicationServlet
import com.vaadin.ui.Alignment
import com.vaadin.ui.Button
import com.vaadin.ui.Component
import com.vaadin.ui.Embedded
import com.vaadin.ui.HorizontalLayout
import com.vaadin.ui.HorizontalSplitPanel
import com.vaadin.ui.Label
import com.vaadin.ui.VerticalLayout
import com.vaadin.ui.Window
import com.vaadin.ui.Button.ClickEvent
import com.vaadin.ui.Button.ClickListener
import com.vaadin.ui.Window.Notification

class AddressBookApplication extends Application implements
	ClickListener, ValueChangeListener, ItemClickListener {


	@WebServlet(urlPatterns='/address-book/*')
	static class Servlet extends AbstractApplicationServlet {
		
		@EJB
		PersonManager personManager

		@Override
		protected Application getNewApplication(HttpServletRequest request) throws ServletException {
			return new AddressBookApplication(personManager)
		}

		@Override
		protected Class<? extends Application> getApplicationClass() throws ClassNotFoundException {
			return AddressBookApplication
		}
	}

	static final Logger logger = LoggerFactory.getLogger(AddressBookApplication)

	final PersonManager personManager
	final NavigationTree tree = new NavigationTree(this)
	final Button newContact = new Button('Add contact')
	final Button search = new Button('Search')
	final Button share = new Button('Share')
	final Button help = new Button('Help')

	final HorizontalSplitPanel horizontalSplit = new HorizontalSplitPanel()

	ListView listView
	SearchView searchView
	PersonList personList
	PersonForm personForm
	HelpWindow helpWindow
	SharingOptions sharingOptions
	PersonReferenceContainer dataSource

	AddressBookApplication(PersonManager personManager) {
		this.personManager = personManager
	}

	@Override
	void init() {
		logger.info('init(): OSGi here I come')
		
//		Person person = personManager.savePerson(new Person(firstName: 'Kirk', lastName: 'Rasmussen'))
//		
//		setMainWindow(new Window('Address Book Demo application'))
//		setTheme('contacts')
//		VerticalLayout layout = new VerticalLayout()
//		layout.addComponent(new Label(caption: "Saved ${person.id}"))
//		getMainWindow().setContent(layout)
		
		dataSource = new PersonReferenceContainer(personManager)
		dataSource.refresh() // Load initial data
		buildMainLayout()
		setMainComponent(getListView())
	}

	@Override
	void buttonClick(ClickEvent event) {
		final Button source = event.getButton()

		if (source == search) {
			showSearchView()
		} else if (source == help) {
			showHelpWindow()
		} else if (source == share) {
			showShareWindow()
		} else if (source == newContact) {
			addNewContanct()
		}
	}

	@Override
	void valueChange(ValueChangeEvent event) {
		Property property = event.getProperty()
		if (property == personList) {
			Person person = personManager.loadPerson((Long) personList.getValue())
			personForm.editContact(person)
		}
	}

	@Override
	void itemClick(ItemClickEvent event) {
		if (event.getSource() == tree) {
			Object itemId = event.getItemId()
			if (itemId != null) {
				if (itemId == NavigationTree.SHOW_ALL) {
					getDataSource().refresh(PersonReferenceContainer.defaultQueryMetaData)
					showListView()
				} else if (itemId == NavigationTree.SEARCH) {
					showSearchView()
				} else if (itemId instanceof SearchFilter) {
					search((SearchFilter) itemId)
				}
			}
		}
	}

	void search(SearchFilter searchFilter) {
		// FIXME
//		Caused by: groovy.lang.MissingMethodException: No signature of method: com.textura.cpms.web.addressbook.data.PersonReferenceContainer.getQueryMetaData() is applicable for argument types: () values: []
		QueryMetaData qmd = new QueryMetaData((String) searchFilter.propertyId, 
				searchFilter.term, 
				dataSource.queryMetaData.orderBy,
				dataSource.queryMetaData.ascending)
		getDataSource().refresh(qmd)
		showListView()

		getMainWindow().showNotification(
				'Searched for ' + searchFilter.getPropertyId() + '='
				+ searchFilter.getTerm() + ', found '
				+ getDataSource().size() + ' item(s).',
				Notification.TYPE_TRAY_NOTIFICATION)
	}

	void saveSearch(SearchFilter searchFilter) {
		tree.addItem(searchFilter)
		tree.setParent(searchFilter, NavigationTree.SEARCH)
		// mark the saved search as a leaf (cannot have children)
		tree.setChildrenAllowed(searchFilter, false)
		// make sure 'Search' is expanded
		tree.expandItem(NavigationTree.SEARCH)
		// select the saved search
		tree.setValue(searchFilter)
	}

	private void buildMainLayout() {
		setMainWindow(new Window('Address Book Demo application'))

		setTheme('contacts')

		VerticalLayout layout = new VerticalLayout()
		layout.setSizeFull()

		layout.addComponent(createToolbar())
		layout.addComponent(horizontalSplit)
		layout.setExpandRatio(horizontalSplit, 1)

		horizontalSplit.setSplitPosition(200, Sizeable.UNITS_PIXELS)
		horizontalSplit.setFirstComponent(tree)

		getMainWindow().setContent(layout)
	}

	private void addNewContanct() {
		showListView()
		personForm.addContact()
	}

	private void showHelpWindow() {
		getMainWindow().addWindow(getHelpWindow())
	}

	private void showShareWindow() {
		getMainWindow().addWindow(getSharingOptions())
	}

	private void showListView() {
		setMainComponent(getListView())
	}

	private void showSearchView() {
		setMainComponent(getSearchView())
	}

	private void setMainComponent(Component c) {
		horizontalSplit.setSecondComponent(c)
	}
	
	/**
	 * View getters exist so we can lazily generate the views, resulting in
	 * faster application startup time.
	 * 
	 * @return the list view.
	 */
	private ListView getListView() {
		if (listView == null) {
			personList = new PersonList(this)
			personForm = new PersonForm(this)
			listView = new ListView(personList, personForm)
		}
		return listView
	}

	private SearchView getSearchView() {
		if (searchView == null) {
			searchView = new SearchView(this)
		}
		return searchView
	}

	private HelpWindow getHelpWindow() {
		if (helpWindow == null) {
			helpWindow = new HelpWindow()
		}
		return helpWindow
	}

	private SharingOptions getSharingOptions() {
		if (sharingOptions == null) {
			sharingOptions = new SharingOptions()
		}
		return sharingOptions
	}

	private HorizontalLayout createToolbar() {
		HorizontalLayout lo = new HorizontalLayout()
		lo.addComponent(newContact)
		lo.addComponent(search)
		lo.addComponent(share)
		lo.addComponent(help)

		search.addListener((ClickListener) this)
		share.addListener((ClickListener) this)
		help.addListener((ClickListener) this)
		newContact.addListener((ClickListener) this)

		search.setIcon(new ThemeResource('icons/32/folder-add.png'))
		share.setIcon(new ThemeResource('icons/32/users.png'))
		help.setIcon(new ThemeResource('icons/32/help.png'))
		newContact.setIcon(new ThemeResource('icons/32/document-add.png'))

		lo.setMargin(true)
		lo.setSpacing(true)

		lo.setStyleName('toolbar')

		lo.setWidth('100%')

		Embedded em = new Embedded('', new ThemeResource('images/logo.png'))
		lo.addComponent(em)
		lo.setComponentAlignment(em, Alignment.MIDDLE_RIGHT)
		lo.setExpandRatio(em, 1)

		return lo
	}
}
