package com.textura.cpms.service.impl;

import org.testng.annotations.DataProvider;
import org.testng.annotations.Test

import com.textura.cpms.service.QueryMetaData;

class PersonQueryBuilderTest {

	static final String[] NATURAL_COL_ORDER = ['firstName', 'lastName', 'email', 'phoneNumber', 'streetAddress', 'postalCode', 'city']
	
	@DataProvider
	Object[][] pqlDataProvider() {
		[
			['''SELECT p.id FROM Person p''',
				new QueryMetaData([] as String[], [] as boolean[]), 
				[]
			],
			['''SELECT p.id,p.firstName FROM Person p''',
				new QueryMetaData([] as String[], [] as boolean[]), 
				['firstName']
			],
			
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p''',
				new QueryMetaData([] as String[], [] as boolean[]), 
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p ORDER BY p.firstName,p.lastName''',
				new QueryMetaData(['firstName', 'lastName'] as String[], [true, true] as boolean[]), 
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p ORDER BY p.firstName,p.lastName DESC''',
				new QueryMetaData(['firstName', 'lastName'] as String[], [true, false] as boolean[]),
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p ORDER BY p.firstName DESC,p.lastName''',
				new QueryMetaData(['firstName', 'lastName'] as String[], [false, true] as boolean[]),
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p ORDER BY p.firstName DESC,p.lastName DESC''',
				new QueryMetaData(['firstName', 'lastName'] as String[], [false, false] as boolean[]),
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p WHERE p.firstName IS NULL''',
				new QueryMetaData('firstName', null, [] as String[], [] as boolean[]),
				NATURAL_COL_ORDER
			],
			['''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p WHERE p.firstName = :searchTerm''',
				new QueryMetaData('firstName', 'TEST', [] as String[], [] as boolean[]),
				NATURAL_COL_ORDER
			]
		] as Object[][]
	}
	
	@Test(enabled = false, dataProvider = 'pqlDataProvider')
	void test(String expected, QueryMetaData queryMetaData, String... propertyNames) {
		String pql = new PersonQueryBuilder(queryMetaData, propertyNames).build()
		assert pql == expected
	}
	
	@Test
	void test() {
		def queryMetaData = new QueryMetaData(['firstName', 'lastName'] as String[], [true, true] as boolean[]) 
		String pql = new PersonQueryBuilder(queryMetaData, NATURAL_COL_ORDER as String[]).build()
		assert pql == '''SELECT p.id,p.firstName,p.lastName,p.email,p.phoneNumber,p.streetAddress,p.postalCode,p.city FROM Person p ORDER BY p.firstName,p.lastName'''
	}
}
